@@ -102,8 +102,7 @@ namespace {
 
         return v;
     }
-    
-}
+} // anonymous namespace
 
 void FileMetaDataProvider::totalPropertyAndInsert(const QString& prop,
                                                   const QList<QVariantMap>& resources,
@@ -111,7 +110,7 @@ void FileMetaDataProvider::totalPropertyAndInsert(const QString& prop,
 {
     if (allProperties.contains(prop)) {
         int total = 0;
-        foreach (const QVariantMap& map, resources) {
+        for (const QVariantMap& map : resources) {
             QVariantMap::const_iterator it = map.constFind(prop);
             if (it == map.constEnd()) {
                 total = 0;
@@ -132,15 +131,16 @@ void FileMetaDataProvider::slotFileFetchFinished(KJob* job)
 {
     FileFetchJob* fetchJob = static_cast<FileFetchJob*>(job);
     QList<QVariantMap> files = fetchJob->data();
-    
+
     Q_ASSERT(!files.isEmpty());
-    
+
     if (files.size() > 1) {
         insertCommonData(files);
     } else {
         m_data = files.first();
         insertSingleFileBasicData();
     }
+    m_readOnly = !fetchJob->canEditAll();
 
     insertEditableData();
     emit loadingFinished();
@@ -150,6 +150,7 @@ void FileMetaDataProvider::slotLoadingFinished(KJob* job)
 {
     IndexedDataRetriever* ret = dynamic_cast<IndexedDataRetriever*>(job);
     m_data = unite(m_data, ret->data());
+    m_readOnly = !ret->canEdit();
 
     emit loadingFinished();
 }
@@ -160,38 +161,40 @@ void FileMetaDataProvider::insertSingleFileBasicData()
     // not work, the modification date needs also to be adjusted...
     Q_ASSERT(m_fileItems.count() <= 1);
     if (m_fileItems.count() == 1) {
-      KFormat format;
       const KFileItem& item = m_fileItems.first();
 
       if (item.isDir()) {
           const int count = subDirectoriesCount(item.url().path());
           if (count == -1) {
-              m_data.insert("kfileitem#size", i18nc("unknown file size", "Unknown"));
+              m_data.insert(QStringLiteral("kfileitem#size"), i18nc("unknown file size", "Unknown"));
           } else {
               const QString itemCountString = i18ncp("@item:intable", "%1 item", "%1 items", count);
-              m_data.insert("kfileitem#size", itemCountString);
+              m_data.insert(QStringLiteral("kfileitem#size"), itemCountString);
           }
       } else {
-          m_data.insert("kfileitem#size", format.formatByteSize(item.size()));
+          KFormat format;
+          m_data.insert(QStringLiteral("kfileitem#size"), format.formatByteSize(item.size()));
       }
 
-      m_data.insert("kfileitem#type", item.mimeComment());
-      m_data.insert("kfileitem#modified", item.time(KFileItem::ModificationTime));
-      m_data.insert("kfileitem#owner", item.user());
-      m_data.insert("kfileitem#group", item.group());
-      m_data.insert("kfileitem#permissions", item.permissionsString());
+      m_data.insert(QStringLiteral("kfileitem#type"), item.mimeComment());
+      m_data.insert(QStringLiteral("kfileitem#modified"), item.time(KFileItem::ModificationTime));
+      QDateTime creationTime = item.time(KFileItem::CreationTime);
+      if (creationTime.isValid()) {
+          m_data.insert(QStringLiteral("kfileitem#created"), creationTime);
+      }
+      m_data.insert(QStringLiteral("kfileitem#accessed"), item.time(KFileItem::AccessTime));
+      m_data.insert(QStringLiteral("kfileitem#owner"), item.user());
+      m_data.insert(QStringLiteral("kfileitem#group"), item.group());
+      m_data.insert(QStringLiteral("kfileitem#permissions"), item.permissionsString());
     }
-
-}     
+}
 
 void FileMetaDataProvider::insertFilesListBasicData()
 {
-  
-    KFormat format;
     // If all directories
     Q_ASSERT(m_fileItems.count() > 1);
     bool allDirectories = true;
-    for (const KFileItem& item : m_fileItems) {
+    for (const KFileItem& item : qAsConst(m_fileItems)) {
         allDirectories &= item.isDir();
         if (!allDirectories) {
             break;
@@ -200,46 +203,36 @@ void FileMetaDataProvider::insertFilesListBasicData()
 
     if (allDirectories) {
         int count = 0;
-        for (const KFileItem& item : m_fileItems) {
+        for (const KFileItem& item : qAsConst(m_fileItems)) {
             count += subDirectoriesCount(item.url().path());
         }
+        const QString itemCountString = i18ncp("@item:intable", "%1 item", "%1 items", count);
+        m_data.insert(QStringLiteral("kfileitem#totalSize"), itemCountString);
 
-        if (count) {
-            const QString itemCountString = i18ncp("@item:intable", "%1 item", "%1 items", count);
-            m_data.insert("kfileitem#totalSize", itemCountString);
-        }
     } else {
         // Calculate the size of all items
         quint64 totalSize = 0;
-        for (const KFileItem& item : m_fileItems) {
+        for (const KFileItem& item : qAsConst(m_fileItems)) {
             if (!item.isDir() && !item.isLink()) {
                 totalSize += item.size();
             }
         }
-        m_data.insert("kfileitem#totalSize", format.formatByteSize(totalSize));
-    }
-}
-
-void FileMetaDataProvider::insertBasicData()
-{
-    if (m_fileItems.count() > 1) {
-      insertFilesListBasicData();
-    } else {
-      insertSingleFileBasicData();
+        KFormat format;
+        m_data.insert(QStringLiteral("kfileitem#totalSize"), format.formatByteSize(totalSize));
     }
 }
 
 void FileMetaDataProvider::insertEditableData()
 {
     if (!m_readOnly) {
-        if (!m_data.contains("tags")) {
-            m_data.insert("tags", QVariant());
+        if (!m_data.contains(QStringLiteral("tags"))) {
+            m_data.insert(QStringLiteral("tags"), QVariant());
         }
-        if (!m_data.contains("rating")) {
-            m_data.insert("rating", 0);
+        if (!m_data.contains(QStringLiteral("rating"))) {
+            m_data.insert(QStringLiteral("rating"), 0);
         }
-        if (!m_data.contains("userComment")) {
-            m_data.insert("userComment", QVariant());
+        if (!m_data.contains(QStringLiteral("userComment"))) {
+            m_data.insert(QStringLiteral("userComment"), QVariant());
         }
     }
 }
@@ -251,25 +244,25 @@ void FileMetaDataProvider::insertCommonData(const QList<QVariantMap>& files)
     //
     QSet<QString> allProperties;
     QList<QVariantMap> propertyList;
-    foreach (const QVariantMap& fileData, files) {
+    for (const QVariantMap& fileData : files) {
         propertyList << fileData;
         allProperties.unite(fileData.uniqueKeys().toSet());
     }
 
     // Special handling for certain properties
-    totalPropertyAndInsert("duration", propertyList, allProperties);
-    totalPropertyAndInsert("characterCount", propertyList, allProperties);
-    totalPropertyAndInsert("wordCount", propertyList, allProperties);
-    totalPropertyAndInsert("lineCount", propertyList, allProperties);
+    totalPropertyAndInsert(QStringLiteral("duration"), propertyList, allProperties);
+    totalPropertyAndInsert(QStringLiteral("characterCount"), propertyList, allProperties);
+    totalPropertyAndInsert(QStringLiteral("wordCount"), propertyList, allProperties);
+    totalPropertyAndInsert(QStringLiteral("lineCount"), propertyList, allProperties);
 
-    foreach (const QString& propUri, allProperties) {
-        foreach (const QVariantMap& map, propertyList) {
+    for (const QString& propUri : qAsConst(allProperties)) {
+        for (const QVariantMap& map : qAsConst(propertyList)) {
             QVariantMap::const_iterator it = map.find(propUri);
             if (it == map.constEnd()) {
                 m_data.remove(propUri);
                 break;
-            } 
-                
+            }
+
             QVariantMap::iterator dit = m_data.find(it.key());
             if (dit == m_data.end()) {
                 m_data.insert(propUri, it.value());
@@ -307,26 +300,30 @@ void FileMetaDataProvider::setFileItem()
     //
     const QUrl url = m_fileItems.first().targetUrl();
     if (!url.isLocalFile()) {
-        insertBasicData();
+        // FIXME - are extended attributes supported for remote files?
+        m_readOnly = true;
+        insertSingleFileBasicData();
         emit loadingFinished();
         return;
     }
 
-    // Not Indexed
+    // Not indexed or only basic file indexing (no content)
     const QString filePath = url.toLocalFile();
-    if (!m_config.fileIndexingEnabled() || !m_config.shouldBeIndexed(filePath)) {
+    if (!m_config.fileIndexingEnabled() || !m_config.shouldBeIndexed(filePath)
+            || m_config.onlyBasicIndexing()) {
         m_realTimeIndexing = true;
 
-        insertBasicData();
+        insertSingleFileBasicData();
         insertEditableData();
 
         IndexedDataRetriever *ret = new IndexedDataRetriever(filePath, this);
-        connect(ret, SIGNAL(finished(KJob*)), this, SLOT(slotLoadingFinished(KJob*)));
+        connect(ret, &IndexedDataRetriever::finished, this, &FileMetaDataProvider::slotLoadingFinished);
         ret->start();
-        
+
+    // Fully indexed by Baloo
     } else {
-        FileFetchJob* job = new FileFetchJob(QStringList() << filePath, this);
-        connect(job, SIGNAL(finished(KJob*)), this, SLOT(slotFileFetchFinished(KJob*)));
+        FileFetchJob* job = new FileFetchJob(QStringList{filePath}, true, this);
+        connect(job, &FileFetchJob::finished, this, &FileMetaDataProvider::slotFileFetchFinished);
         job->start();
     }
 }
@@ -338,23 +335,28 @@ void FileMetaDataProvider::setFileItems()
     //   * Indexed
 
     QStringList urls;
-    // Only extract data from indexed files, 
+    urls.reserve(m_fileItems.size());
+    // Only extract data from indexed files,
     // it would be too expensive otherwise.
-    Q_FOREACH (const KFileItem& item, m_fileItems) {
+    for (const KFileItem& item : qAsConst(m_fileItems)) {
         const QUrl url = item.targetUrl();
         if (url.isLocalFile()) {
             urls << url.toLocalFile();
         }
     }
 
+    insertFilesListBasicData();
     if (!urls.isEmpty()) {
-        insertBasicData();
-        
-        FileFetchJob* job = new FileFetchJob(urls, this);
-        connect(job, SIGNAL(finished(KJob*)), this, SLOT(slotFileFetchFinished(KJob*)));
+        // Editing only if all URLs are local
+        bool canEdit = (urls.size() == m_fileItems.size());
+
+        FileFetchJob* job = new FileFetchJob(urls, canEdit, this);
+        connect(job, &FileFetchJob::finished, this, &FileMetaDataProvider::slotFileFetchFinished);
         job->start();
+
     } else {
-        insertBasicData();
+        // FIXME - are extended attributes supported for remote files?
+        m_readOnly = true;
         emit loadingFinished();
     }
 }
@@ -376,39 +378,25 @@ void FileMetaDataProvider::setItems(const KFileItemList& items)
 
 QString FileMetaDataProvider::label(const QString& metaDataLabel) const
 {
-    struct TranslationItem {
-        const char* const key;
-        const char* const context;
-        const char* const value;
+    static QHash<QString, QString> hash = {
+        { QStringLiteral("kfileitem#comment"), i18nc("@label", "Comment") },
+        { QStringLiteral("kfileitem#created"), i18nc("@label", "Created") },
+        { QStringLiteral("kfileitem#accessed"), i18nc("@label", "Accessed") },
+        { QStringLiteral("kfileitem#modified"), i18nc("@label", "Modified") },
+        { QStringLiteral("kfileitem#owner"), i18nc("@label", "Owner") },
+        { QStringLiteral("kfileitem#group"), i18nc("@label", "Group") },
+        { QStringLiteral("kfileitem#permissions"), i18nc("@label", "Permissions") },
+        { QStringLiteral("kfileitem#rating"), i18nc("@label", "Rating") },
+        { QStringLiteral("kfileitem#size"), i18nc("@label", "Size") },
+        { QStringLiteral("kfileitem#tags"), i18nc("@label", "Tags") },
+        { QStringLiteral("kfileitem#totalSize"), i18nc("@label", "Total Size") },
+        { QStringLiteral("kfileitem#type"), i18nc("@label", "Type") },
+        { QStringLiteral("tags"), i18nc("@label", "Tags") },
+        { QStringLiteral("rating"), i18nc("@label", "Rating") },
+        { QStringLiteral("userComment"), i18nc("@label", "Comment") },
+        { QStringLiteral("originUrl"), i18nc("@label", "Downloaded From") },
     };
 
-    static const TranslationItem translations[] = {
-        { "kfileitem#comment", I18N_NOOP2_NOSTRIP("@label", "Comment") },
-        { "kfileitem#modified", I18N_NOOP2_NOSTRIP("@label", "Modified") },
-        { "kfileitem#owner", I18N_NOOP2_NOSTRIP("@label", "Owner") },
-        { "kfileitem#group", I18N_NOOP2_NOSTRIP("@label", "Group") },
-        { "kfileitem#permissions", I18N_NOOP2_NOSTRIP("@label", "Permissions") },
-        { "kfileitem#rating", I18N_NOOP2_NOSTRIP("@label", "Rating") },
-        { "kfileitem#size", I18N_NOOP2_NOSTRIP("@label", "Size") },
-        { "kfileitem#tags", I18N_NOOP2_NOSTRIP("@label", "Tags") },
-        { "kfileitem#totalSize", I18N_NOOP2_NOSTRIP("@label", "Total Size") },
-        { "kfileitem#type", I18N_NOOP2_NOSTRIP("@label", "Type") },
-        { "tags", I18N_NOOP2_NOSTRIP("@label", "Tags") },
-        { "rating", I18N_NOOP2_NOSTRIP("@label", "Rating") },
-        { "userComment", I18N_NOOP2_NOSTRIP("@label", "Comment") },
-        { "originUrl", I18N_NOOP2_NOSTRIP("@label", "Downloaded From") },
-        { nullptr, nullptr, nullptr} // Mandatory last entry
-    };
-
-    static QHash<QString, QString> hash;
-    if (hash.isEmpty()) {
-        const TranslationItem* item = &translations[0];
-        while (item->key != nullptr) {
-            hash.insert(item->key, i18nc(item->context, item->value));
-            ++item;
-        }
-    }
-
     QString value = hash.value(metaDataLabel);
     if (value.isEmpty()) {
         value = KFileMetaData::PropertyInfo::fromName(metaDataLabel).displayName();
@@ -419,45 +407,61 @@ QString FileMetaDataProvider::label(const QString& metaDataLabel) const
 
 QString FileMetaDataProvider::group(const QString& label) const
 {
-    static QHash<QString, QString> uriGrouper;
-    if (uriGrouper.isEmpty()) {
+    static QHash<QString, QString> uriGrouper = {
+
         // KFileItem Data
-        uriGrouper.insert(QLatin1String("kfileitem#type"), QLatin1String("0FileItemA"));
-        uriGrouper.insert(QLatin1String("kfileitem#size"), QLatin1String("0FileItemB"));
-        uriGrouper.insert(QLatin1String("kfileitem#totalSize"), QLatin1String("0FileItemB"));
-        uriGrouper.insert(QLatin1String("kfileitem#modified"), QLatin1String("0FileItemC"));
-        uriGrouper.insert(QLatin1String("kfileitem#owner"), QLatin1String("0FileItemD"));
-        uriGrouper.insert(QLatin1String("kfileitem#group"), QLatin1String("0FileItemE"));
-        uriGrouper.insert(QLatin1String("kfileitem#permissions"), QLatin1String("0FileItemF"));
+        { QStringLiteral("kfileitem#type"), QStringLiteral("0FileItemA") },
+        { QStringLiteral("kfileitem#size"), QStringLiteral("0FileItemB") },
+        { QStringLiteral("kfileitem#totalSize"), QStringLiteral("0FileItemB") },
+        { QStringLiteral("kfileitem#modified"), QStringLiteral("0FileItemC") },
+        { QStringLiteral("kfileitem#accessed"), QStringLiteral("0FileItemD") },
+        { QStringLiteral("kfileitem#created"), QStringLiteral("0FileItemE") },
+        { QStringLiteral("kfileitem#owner"), QStringLiteral("0FileItemF") },
+        { QStringLiteral("kfileitem#group"), QStringLiteral("0FileItemG") },
+        { QStringLiteral("kfileitem#permissions"), QStringLiteral("0FileItemH") },
 
         // Editable Data
-        uriGrouper.insert(QLatin1String("tags"), QLatin1String("1EditableDataA"));
-        uriGrouper.insert(QLatin1String("rating"), QLatin1String("1EditableDataB"));
-        uriGrouper.insert(QLatin1String("userComment"), QLatin1String("1EditableDataC"));
+        { QStringLiteral("tags"), QStringLiteral("1EditableDataA") },
+        { QStringLiteral("rating"), QStringLiteral("1EditableDataB") },
+        { QStringLiteral("userComment"), QStringLiteral("1EditableDataC") },
 
         // Image Data
-        uriGrouper.insert(QLatin1String("width"), QLatin1String("2SizA"));
-        uriGrouper.insert(QLatin1String("height"), QLatin1String("2SizeB"));
+        { QStringLiteral("width"), QStringLiteral("2ImageA") },
+        { QStringLiteral("height"), QStringLiteral("2ImageB") },
+        { QStringLiteral("photoFNumber"), QStringLiteral("2ImageC") },
+        { QStringLiteral("photoExposureTime"), QStringLiteral("2ImageD") },
+        { QStringLiteral("photoExposureBiasValue"), QStringLiteral("2ImageE") },
+        { QStringLiteral("photoISOSpeedRatings"), QStringLiteral("2ImageF") },
+        { QStringLiteral("photoFocalLength"), QStringLiteral("2ImageG") },
+        { QStringLiteral("photoFocalLengthIn35mmFilm"), QStringLiteral("2ImageH") },
+        { QStringLiteral("photoFlash"), QStringLiteral("2ImageI") },
+        { QStringLiteral("imageOrientation"), QStringLiteral("2ImageJ") },
+        { QStringLiteral("photoGpsLatitude"), QStringLiteral("2ImageK") },
+        { QStringLiteral("photoGpsLongitude"), QStringLiteral("2ImageL") },
+        { QStringLiteral("photoGpsAltitude"), QStringLiteral("2ImageM") },
+        { QStringLiteral("manufacturer"), QStringLiteral("2ImageN") },
+        { QStringLiteral("model"), QStringLiteral("2ImageO") },
 
-        // Music Data
-        uriGrouper.insert("title", QLatin1String("3MusicA"));
-        uriGrouper.insert("artist", QLatin1String("3MusicB"));
-        uriGrouper.insert("album", QLatin1String("3MusicC"));
-        uriGrouper.insert("genre", QLatin1String("3MusicD"));
-        uriGrouper.insert("trackNumber", QLatin1String("3MusicE"));
-
-        // Audio Data
-        uriGrouper.insert("duration", QLatin1String("4AudioA"));
-        uriGrouper.insert("sampleRate", QLatin1String("4AudioB"));
-        uriGrouper.insert("sampleCount", QLatin1String("4AudioC"));
+        // Media Data
+        { QStringLiteral("title"), QStringLiteral("3MediaA") },
+        { QStringLiteral("artist"), QStringLiteral("3MediaB") },
+        { QStringLiteral("album"), QStringLiteral("3MediaC") },
+        { QStringLiteral("albumArtist"), QStringLiteral("3MediaD") },
+        { QStringLiteral("genre"), QStringLiteral("3MediaE") },
+        { QStringLiteral("trackNumber"), QStringLiteral("3MediaF") },
+        { QStringLiteral("discNumber"), QStringLiteral("3MediaG") },
+        { QStringLiteral("releaseYear"), QStringLiteral("3MediaH") },
+        { QStringLiteral("duration"), QStringLiteral("3MediaI") },
+        { QStringLiteral("sampleRate"), QStringLiteral("3MediaJ") },
+        { QStringLiteral("bitRate"), QStringLiteral("3MediaK") },
 
         // Miscellaneous Data
-        uriGrouper.insert("originUrl", QLatin1String("5MiscA"));
-    }
+        { QStringLiteral("originUrl"), QStringLiteral("4MiscA") },
+    };
 
     const QString val = uriGrouper.value(label);
     if (val.isEmpty()) {
-        return "lastGroup";
+        return QStringLiteral("lastGroup");
     }
     return val;
 }
@@ -492,7 +496,7 @@ int FileMetaDataProvider::subDirectoriesCount(const QString& path)
     // Copyright (C) 2006 David Faure <faure@kde.org>
 
     int count = -1;
-    DIR* dir = ::opendir(QFile::encodeName(path));
+    DIR* dir = ::opendir(QFile::encodeName(path).constData());
     if (dir) {
         count = 0;
         struct dirent *dirEntry = nullptr;
